# -*- coding: utf-8 -*-
"""
File:
    themis_load.py

Description:
    Data loading functions for the THEMIS mission.

Example:
    pyspedas.themis_load(['2015-12-31'], ['tha'], '*', ['l2'])

Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31']).
    probes: str/list of str
        Probes for missions with multiple probes (eg. ['tha', 'thb']),
        wildcard ('*') for all probes.
    instruments: str/list of str
        List of instruments (eg. ['fft']), wildcard ('*') for all instruments.
    level: str
        Either 'l2' or 'l1'.
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only,
        if False then they are also loaded into pytplot using cdf_to_tplot.
    varformat : str
        The file variable formats to load into tplot.
        Wildcard character "*" is accepted.
        By default, all variables are loaded in.
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.
        By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.
        By default, no suffix is added.

Notes:
    List of possible values for "probes" variable:
    ['tha', 'tha', 'thc', 'thd', 'the']

    List of possible values for L2 themis "instruments" variable:
    ['efi', 'esa', 'fbk', 'fft', 'fgm', 'fit', 'gmom', 'mom', 'scm', 'sst']

    List of possible values for L1 themis "instruments" variable:
    ['bau', 'eff', 'efp', 'efw', 'esa', 'fbk', 'fff_16', 'fff_32', 'fff_64',
    'ffp_16', 'ffp_32', 'ffp_64', 'ffw_16', 'ffw_32', 'ffw_64', 'fgm', 'fit',
    'hsk', 'mom', 'scf', 'scm', 'scmode', 'scp', 'scw', 'spin', 'sst', 'state',
    'trg', 'vaf', 'vap', 'vaw']
"""

import os
import pyspedas
from .themis_helpers import get_instruments, get_probes


def themis_filename(dates, probes, instruments, level):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'themis_remote' in prefs:
        remote_path = prefs['themis_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs:
        data_dir = prefs['data_dir'] + 'themis'

    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    if level != 'l1':
        level = 'l2'
    probes = get_probes(probes)
    instruments = get_instruments(instruments, level)
    dates = pyspedas.get_dates(dates)

    if level == 'l1':
        version = '?'
    else:
        version = '1'

    file_list = []
    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]
        day = sdate[8:10]
        for probe in probes:
            for instrument in instruments:
                # file_dir = 'tha/l2/fgm/2015/'
                file_dir = probe + '/' + level + '/' + instrument + '/' + year
                file_dir_local = os.path.join(probe, level, instrument, year)
                # filename = 'tha_l2_fgm_20150101_v01.cdf'
                filename = probe + '_' + level + '_' + instrument + '_'\
                    + year + month + day + '_v0' + version + '.cdf'

                remote_file = remote_path + '/' + file_dir + '/' + filename
                local_file = os.path.join(data_dir, file_dir_local, filename)

                file_list.append((remote_file, local_file))

    return file_list


def themis_load(dates, probes, instruments, level, downloadonly=False,
                varformat=None, get_support_data=False, prefix='', suffix=''):
    """Loads themis data into pytplot variables"""

    file_list = themis_filename(dates, probes, instruments, level)
    # print(file_list)
    count = 0
    dcount = 0

    for remotef, localf in file_list:
        count += 1
        resp, err, localfile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + localfile)
            dcount += 1
            if not downloadonly:
                try:
                    cdfvars = pyspedas.cdf_to_tplot(localfile, varformat,
                                                    get_support_data, prefix,
                                                    suffix, False, True)

                except TypeError as e:
                    msg = "cdf_to_tplot could not load " + localfile
                    + "\nError:\n" + str(e)
                    print(msg)

        else:
            print(str(count) + '. There was a problem. Could not download \
                  file: ' + remotef)
            print(err)

    if len(dates) == 2:
        pyspedas.time_clip(cdfvars, dates[0], dates[1], cdfvars)

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pyspedas.tplot_names())



# -*- coding: utf-8 -*-
"""
File:
    ex_gmag.py

Description:
    Basic example with THEMIS GMAG data.
    Downloads THEMIS data from EPO GMAG stations and plots it.

"""

import pyspedas
import pytplot


def ex_gmag():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Define list of dates
    time_list = ['2015-12-31']
    # Get a list of EPO gmag stations
    sites = pyspedas.gmag_list(group='epo')
    # Download gmag files and load data into pytplot variables
    pyspedas.load_data('gmag', time_list, sites, '', '')
    # Get a list of loaded sites
    sites_loaded = pyspedas.tplot_names()
    # Subtract mean values
    pyspedas.subtract_average(sites_loaded, '')
    # Download AE index data
    pyspedas.load_data('gmag', time_list, ['idx'], '', '')
    # Plot
    sites_loaded = pyspedas.tplot_names()
    pytplot.tplot_options('title', 'EPO GMAG 2015-12-31')
    pytplot.tplot(sites_loaded, bokeh=True)

# Run the example code
# ex_gmag()



# -*- coding: utf-8 -*-
"""
File:
    download_file_utilities.py

Description:
    Functions used by maven_load.
"""

uname = ''
pword = ''


def get_filenames(query, public):
    import urllib

    public_url = 'https://lasp.colorado.edu/maven/sdc/public/files/api/v1/search/science/fn_metadata/file_names' + \
                 '?' + query
    private_url = 'https://lasp.colorado.edu/maven/sdc/service/files/api/v1/search/science/fn_metadata/file_names' + \
                  '?' + query
    if not public:
        username = uname
        password = pword
        p = urllib.request.HTTPPasswordMgrWithDefaultRealm()
        p.add_password(None, private_url, username, password)
        handler = urllib.request.HTTPBasicAuthHandler(p)
        opener = urllib.request.build_opener(handler)
        urllib.request.install_opener(opener)
        page = urllib.request.urlopen(private_url)
    else:
        page = urllib.request.urlopen(public_url)

    return page.read().decode("utf-8")


def get_file_from_site(filename, public, data_dir):
    import os
    import urllib

    public_url = 'https://lasp.colorado.edu/maven/sdc/public/files/api/v1/search/science/fn_metadata/download' + '?file=' + filename
    private_url = 'https://lasp.colorado.edu/maven/sdc/service/files/api/v1/search/science/fn_metadata/download' + '?file=' + filename

    if not public:
        username = uname
        password = pword
        p = urllib.request.HTTPPasswordMgrWithDefaultRealm()
        p.add_password(None, private_url, username, password)
        handler = urllib.request.HTTPBasicAuthHandler(p)
        opener = urllib.request.build_opener(handler)
        urllib.request.install_opener(opener)
        page = urllib.request.urlopen(private_url)
    else:
        page = urllib.request.urlopen(public_url)

    with open(os.path.join(data_dir, filename), "wb") as code:
        code.write(page.read())

    return


def get_orbit_files():
    import os
    import urllib
    import re

    orbit_files_url = "http://naif.jpl.nasa.gov/pub/naif/MAVEN/kernels/spk/"
    pattern = 'maven_orb_rec(\.orb|.{17}\.orb)'
    page = urllib.request.urlopen(orbit_files_url)
    page_string = str(page.read())
    full_path = os.path.realpath(__file__)
    toolkit_path = os.path.dirname(full_path)

    orbit_files_path = os.path.join(toolkit_path, "orbitfiles")

    if not os.path.exists(orbit_files_path):
        os.mkdir(orbit_files_path)

    for matching_pattern in re.findall(pattern, page_string):
        filename = "maven_orb_rec" + matching_pattern
        o_file = urllib.request.urlopen(orbit_files_url + filename)
        with open(os.path.join(orbit_files_path, filename), "wb") as code:
            code.write(o_file.read())

    merge_orbit_files()

    return


def merge_orbit_files():
    import os
    import re

    full_path = os.path.realpath(__file__)
    toolkit_path = os.path.dirname(full_path)
    orbit_files_path = os.path.join(toolkit_path, "orbitfiles")
    pattern = 'maven_orb_rec(_|)(|.{6})(|_.{9}).orb'
    orb_dates = []
    orb_files = []
    for f in os.listdir(orbit_files_path):
        x = re.match(pattern, f)
        if x is not None:
            orb_files.append(os.path.join(orbit_files_path, f))
            if x.group(2) != '':
                orb_dates.append(x.group(2))
            else:
                orb_dates.append('999999')

    sorted_files = [x for (y, x) in sorted(zip(orb_dates, orb_files))]

    with open(os.path.join(toolkit_path, 'maven_orb_rec.orb'), "w") as code:
        skip_2_lines = False
        for o_file in sorted_files:
            with open(o_file) as f:
                if skip_2_lines:
                    f.readline()
                    f.readline()
                skip_2_lines = True
                code.write(f.read())

    return


def get_access():
    import os
    toolkit_path = os.path.dirname(__file__)
    with open(os.path.join(toolkit_path, 'access.txt'), 'r') as f:
        f.readline()
        s = f.readline().rstrip()
        s = s.split(' ')
        if s[1] == '1':
            return False
        else:
            return True


def get_root_data_dir():
    import pyspedas
    # Get preferred data download location for pyspedas project
    prefs = pyspedas.get_spedas_prefs()
    if 'data_dir' in prefs:
        download_path = prefs['data_dir']
    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    import os
    # Get the "toolkit path" (where MAVEN download code is)
    full_path = os.path.realpath(__file__)
    toolkit_path = os.path.dirname(full_path)
    if not os.path.exists(os.path.join(toolkit_path, 'mvn_toolkit_prefs.txt')):
        create_pref_file(toolkit_path, download_path)

    with open(os.path.join(toolkit_path, 'mvn_toolkit_prefs.txt'), 'r') as f:
        f.readline()
        s = f.readline().rstrip()
        # Get rid of first space
        s = s.split(' ')
        nothing = ' '
    return nothing.join(s[1:])


def create_pref_file(toolkit_path, download_path):
    import os

    # Put data download path into preferences file
    with open(os.path.join(toolkit_path, 'mvn_toolkit_prefs.txt'), 'w') as f:
        f.write("'; IDL Toolkit Data Preferences File'\n")
        f.write('mvn_root_data_dir: ' + download_path)

    return


def set_new_data_root_dir():
    import os

    # Get new preferred data download location for pyspedas project
    valid_path = input("Enter directory preference: ")
    while not os.path.exists(valid_path):
        valid_path = input("Specified path does not exist. Enter new path: ")
    download_path = valid_path
    print("root_data_dir set to " + download_path)

    # Edit the pyspedas pref file to reflect the new data download location
    pref_file = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'spd_prefs_txt.py'))
    with open(pref_file, 'r') as f:
        content = f.readlines()
    content[3] = 'data_dir = \'{}\'\n'.format(download_path)

    with open(pref_file, 'w') as f:
        for line in content:
            f.write(line)

    # Also edit the mvn_toolkit_prefs file to reflect the new data download location
    full_path = os.path.realpath(__file__)
    toolkit_path = os.path.dirname(full_path)
    create_pref_file(toolkit_path, download_path)


def get_new_files(files_on_site, data_dir, instrument, level):
    import os
    import re

    fos = files_on_site
    files_on_hd = []
    for (dir, _, files) in os.walk(data_dir):
        for f in files:
            if re.match('mvn_' + instrument + '_' + level + '_*', f):
                files_on_hd.append(f)

    x = set(files_on_hd).intersection(files_on_site)
    for matched_file in x:
        fos.remove(matched_file)

    return fos


def create_dir_if_needed(f, data_dir, level):
    import os

    if level == 'insitu':
        year, month, _ = get_year_month_day_from_kp_file(f)
    else:
        year, month, _ = get_year_month_day_from_sci_file(f)

    if not os.path.exists(os.path.join(data_dir, year, month)):
        os.makedirs(os.path.join(data_dir, year, month))

    full_path = os.path.join(data_dir, year, month)

    return full_path


def get_year_month_day_from_kp_file(f):
    date_string = f.split('_')[3]
    year = date_string[0:4]
    month = date_string[4:6]
    day = date_string[6:8]

    return year, month, day


def get_year_month_day_from_sci_file(f):
    date_string = f.split('_')[4]
    year = date_string[0:4]
    month = date_string[4:6]
    day = date_string[6:8]

    return year, month, day


def display_progress(x, y):
    num_stars = int(round(float(x) / y * 70))
    print("||" + "*" * num_stars + "-" * (70 - num_stars) + "||" + " ( " + str(round(100 * float(x) / y)) + "% )")
    return


def get_uname_and_password():
    global uname
    global pword
    import getpass

    uname = input("Enter user name to access the team website: ")
    pword = getpass.getpass("Enter your password: ")
    return



# -*- coding: utf-8 -*-
"""
File:
    orbit_time.py

Description:
    MAVEN orbit file handling routines.
"""

import os


def month_to_num(month_string):

    month_to_num_dict = {'JAN': '01', 'FEB': '02', 'MAR': '03', 'APR': '04', 'MAY': '05', 'JUN': '06', 'JUL': '07',
                         'AUG': '08', 'SEP': '09', 'OCT': '10', 'NOV': '11', 'DEC': '12'}

    if month_string in month_to_num_dict.keys():
        return month_to_num_dict[month_string]
    else:
        raise ValueError('Month string is not valid')


def orbit_time(begin_orbit, end_orbit=None):
    orb_file = os.path.join(os.path.dirname(__file__),
                            'maven_orb_rec.orb')

    with open(orb_file, "r") as f:
        if end_orbit is None:
            end_orbit = begin_orbit
        orbit_num = []
        time = []
        f.readline()
        f.readline()
        for line in f:
            line = line[0:28]
            line = line.split(' ')
            line = [x for x in line if x != '']
            orbit_num.append(int(line[0]))
            time.append(line[1] + "-" + month_to_num(line[2]) + "-" + line[3] + "T" + line[4])
        try:

            if orbit_num.index(begin_orbit) > len(time) or orbit_num.index(end_orbit) + 1 > len(time):
                print("Orbit numbers not found.  Please choose a number between 1 and %s.", orbit_num[-1])
                return [None, None]
            else:
                begin_time = time[orbit_num.index(begin_orbit)]
                end_time = time[orbit_num.index(end_orbit) + 1]
        except ValueError:
            return [None, None]
    return [begin_time, end_time]



# -*- coding: utf-8 -*-
"""
File:
    ex_create.py

Description:
    Creates a new pytplot object and applies analysis functions.

"""

import pyspedas
import pytplot
import numpy


def ex_create():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Create a sin wave plot
    a = list(range(0, 101))
    b = [2.0 / 100.0 * numpy.pi * s for s in a]
    c = pyspedas.time_float('2017-01-01')
    x = list()
    y = list()
    for i in range(len(b)):
        x.append(c + 60.0 / (2 * numpy.pi) * 60.0 * b[i])
        y.append(1000.0 * numpy.sin(b[i]))

    # Store data
    pytplot.store_data('sinx', data={'x': x, 'y': y})
    # Apply tclip
    pyspedas.tclip('sinx', -800.0, 800.0)
    # Remove NaN values
    pyspedas.tdeflag('sinx-clip')
    # Interpolate
    pyspedas.tinterpol(['sinx-clip-deflag'], ['sinx'], 'quadratic')
    # Plot
    pytplot.ylim('sinx', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip-deflag', -1100.0, 1100.0)
    pytplot.ylim('sinx-clip-deflag-itrp', -1100.0, 1100.0)
    pytplot.tplot_options('title', 'Interpolation example')
    pytplot.tplot(['sinx', 'sinx-clip', 'sinx-clip-deflag',
                   'sinx-clip-deflag-itrp'])

# Run the example code
# ex_create()



# -*- coding: utf-8 -*-
"""
File:
    omni_load.py

Description:
    Data loading functions for OMNI.

Example:
    pyspedas.omni_load('2015-12-31', '5min', False)

Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31']).
    level: str
        Either '1min' or '5min'.
    downloadonly: bool (True/False)
        If True then CDF files are downloaded only,
        if False then they are also loaded into pytplot using cdf_to_tplot.
    varformat : str
        The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.
        By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.
        By default, no suffix is added.

Notes:
    https://cdaweb.gsfc.nasa.gov/istp_public/data/omni/hro_1min/2017/
    omni_hro_1min_20171201_v01.cdf
"""

import os
import pytplot
import pyspedas


def omni_filename(dates, level):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'omni_remote' in prefs:
        remote_path = prefs['omni_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs:
        data_dir = prefs['data_dir'] + 'omni'

    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    if level != '1min':
        level = '5min'

    dates = pyspedas.get_dates(dates)
    version = '?'

    file_list = []
    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]

        file_dir = r'hro_' + level + '/' + year
        file_dir_local = os.path.join('hro_' + level, year)
        filename = r'omni_hro_' + level + '_' + year + month + '01' + '_v0' \
                                + version + '.cdf'

        remote_file = remote_path + '/' + file_dir + '/' + filename
        local_file = os.path.join(data_dir, file_dir_local, filename)

        if len([item for item in file_list if item[0] == remote_file]) == 0:
            file_list.append((remote_file, local_file))

    return file_list


def omni_load(dates, level, downloadonly=False, varformat=None,
              get_support_data=False, prefix='', suffix=''):
    """Loads OMNI data into pytplot variables"""

    file_list = omni_filename(dates, level)

    # 1. Download files
    count = 0
    downloaded_files = []
    for remotef, localf in file_list:
        count += 1
        resp, err, localfile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + localfile)
            downloaded_files.append(localfile)
        else:
            print(str(count) + '. Error: Could not download file: ' + remotef)
            print(err)
    print('Downloaded ' + str(len(downloaded_files)) + ' files.')

    # 2. Load files into tplot
    downloaded_vars = []
    if not downloadonly:
        try:
            downloaded_vars = pytplot.cdf_to_tplot(downloaded_files, varformat,
                                                   get_support_data, prefix,
                                                   suffix, False, True)
        except TypeError as e:
            msg = "cdf_to_tplot could not load all data.\nError:\n" + str(e)
            print(msg)
    print('Loaded ' + str(len(downloaded_vars)) + ' variables.')

    # 3. Time clip
    if len(downloaded_vars) > 0:
        pyspedas.time_clip(downloaded_vars, dates[0], dates[1], '')



# -*- coding: utf-8 -*-
"""
File:
    ex_omni.py

Description:
    Basic example of plotting OMNI data.

"""

import pyspedas
import pytplot


def ex_omni():
    # Print the installed version of pyspedas
    pyspedas.version()
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download OMNI data for 2015-12-31
    pyspedas.load_data('omni', ['2015-12-31 00:00:00',
                                '2016-01-01 23:59:59'], '', '', '1min')

    # Plot
    pytplot.tplot_options('title', 'OMNI flow_speed 2015-12-31 to 2016-01-01')
    pytplot.tplot(['flow_speed'])

# Run the example code
# ex_omni()



# -*- coding: utf-8 -*-
"""
File:
    ex_analysis.py

Description:
    Basic example using analysis functions.
    Downloads THEMIS data and plots it.

"""

import pyspedas
import pytplot


def ex_analysis():
    # Print the installed version of pyspedas
    pyspedas.version()
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download THEMIS state data for 2015-12-31
    pyspedas.load_data('themis', '2015-12-31', ['tha'], 'state', 'l1')
    # Use some analysis functions on tplot variables
    pyspedas.subtract_average('tha_pos')
    pyspedas.subtract_median('tha_pos')
    # Plot
    pytplot.tplot(["tha_pos", "tha_pos-d", "tha_pos-m"])

# Run the example code
# ex_analysis()



'''

This crib sheet shows how to load and plot ASPOC data

'''

from pyspedas import mms_load_aspoc
from pytplot import tplot

# load ASPOC data 
mms_load_aspoc(trange=['2015-10-16', '2015-10-17'], data_rate='srvy')

# plot the ion current from aspoc1, aspoc2, and total current
tplot(['mms1_asp1_ionc', 'mms1_asp2_ionc', 'mms1_aspoc_ionc'])



# -*- coding: utf-8 -*-
"""
File:
    gmag_load.py

Description:
    Data loading functions for the THEMIS GMAG stations.

Example:
    pyspedas.gmag_load(['2015-12-31'], ['bmls', 'ccnv', 'drby'])

Parameters:
    dates: str/list of str ['yyyy-mm-dd']
        List of dates to be downloaded (eg. ['2015-12-31']).
    stations: str/list of str
        GMAG station names (eg. 'bmls').
    group: str
        GMAG group of stations (eg. 'epo').
        If specified, then stations is ignored.
    downloadonly: bool
        If True then CDF files are downloaded only, if False
        then they are also loaded into pytplot using pytplot.cdf_to_tplot.
    varformat : str
        The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.
    get_support_data: bool
        Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".
    prefix: str
        The tplot variable names will be given this prefix.
        By default, no prefix is added.
    suffix: str
        The tplot variable names will be given this suffix.
        By default, no suffix is added.

Notes:
    For gmag station names and groups, see:
        http://themis.ssl.berkeley.edu/gmag/gmag_list.php
"""


import requests
import os
import pyspedas

gmag_dict = {}


def query_gmags():
    """ returns a dictionary of gmag stations and all their metadata
    """

    url = 'http://themis.ssl.berkeley.edu/gmag/gmag_json.php'
    global gmag_dict

    params = dict(
        station='',
        group=''
    )
    resp = requests.get(url=url, params=params)
    data = resp.json()
    gmag_dict = data

    return data


def get_group(station_name):
    """ returns a list with the groups that station belongs to
    """

    global gmag_dict
    group_list = []
    if gmag_dict == {}:
        gmag_dict = query_gmags()

    for station in gmag_dict:
        if station['ccode'].lower() == station_name:
            for key, value in station.items():
                if value == 'Y':
                    if station['ccode'].lower() not in group_list:
                        group_list.append(key)

    return group_list


def gmag_list(group='all'):
    """ returns a list of stations
        prints a list of "stations: start date - end date"
    """

    global gmag_dict
    station_list = []
    if gmag_dict == {}:
        gmag_dict = query_gmags()

    for station in gmag_dict:
        station_name = station['ccode'].lower()
        station_group = get_group(station_name)
        if group in ['all', '*', ''] or group in station_group:
            station_list.append(station_name)
            print(station_name + ": from " + station['day_first'] + " to "
                  + station['day_last'])

    return station_list


def gmag_groups():
    """ returns a dictionary of station groups with a list of stations
        prints a list of "group:'stations'"
    """

    global gmag_dict
    group_dict = {}
    if gmag_dict == {}:
        gmag_dict = query_gmags()

    for station in gmag_dict:
        for key, value in station.items():
            if value == 'Y':
                if key in group_dict:
                    if station['ccode'].lower() not in group_dict[key]:
                        group_dict[key].append(station['ccode'].lower())
                else:
                    group_dict[key] = []
                    group_dict[key].append(station['ccode'].lower())

    # print them
    print()
    for g, s in group_dict.items():
        print(g + ":" + ",'".join(s) + "'")

    return group_dict


def check_gmag(station_name):
    """ returns 1 if station_name is in the gmag list, 0 otherwise"
    """

    global gmag_dict
    if gmag_dict == {}:
        gmag_dict = query_gmags()

    for station in gmag_dict:
        if station['ccode'].lower() == station_name.lower():
            return 1

    return 0


def check_greenland(station_name):
    """ returns 1 if station_name is in the greenland gmag list, 0 otherwise"
    """

    global gmag_dict
    if gmag_dict == {}:
        gmag_dict = query_gmags()

    for station in gmag_dict:
        if station['ccode'].lower() == station_name.lower():
            if ((station['country'] is not None
                 and station['country'].lower() == 'greenland')
                or (station['greenland'] is not None
                    and station['greenland'].lower() == 'y')):
                return 1

    return 0


def gmag_filename(dates, stations):
    """Create a list of tuples for downloading: remote_file, local_file"""
    prefs = pyspedas.get_spedas_prefs()
    if 'themis_remote' in prefs:
        remote_path = prefs['themis_remote']
    else:
        raise NameError('remote_path is not found in spd_prefs.txt')
    if 'data_dir' in prefs:
        data_dir = prefs['data_dir']

    else:
        raise NameError('data_dir is not found in spd_prefs.txt')

    dates = pyspedas.get_dates(dates)

    file_list = []

    probe = 'thg'
    level = 'l2'
    instrument = 'mag'
    version = '1'
    if stations[0] == 'idx':
        level = 'l1'

    for sdate in dates:
        year = sdate[0:4]
        month = sdate[5:7]
        day = sdate[8:10]
        for station in stations:
            # file_dir = 'tha/l2/fgm/2015/'
            if station == 'idx':
                level = 'l1'
                file_dir = probe + '/' + level + '/' + instrument + '/' \
                    + station + '/' + year
                filename = probe + '_' + level + '_' + station + '_' + year \
                    + month + day + '_v0' + version + '.cdf'
            elif check_greenland(station):
                # thg/greenland_gmag/l2
                file_dir = probe + '/greenland_gmag/' + level + '/' \
                    + station + '/' + year
                filename = probe + '_' + level + '_' + instrument + '_' \
                    + station + '_' + year + month + day + '_v0' \
                    + version + '.cdf'
            else:
                # thg/l2/mag/
                file_dir = probe + '/' + level + '/' + instrument + '/' \
                    + station + '/' + year
                filename = probe + '_' + level + '_' + instrument + '_' \
                    + station + '_' + year + month + day + '_v0' \
                    + version + '.cdf'

            file_dir_local = os.path.join(probe, level, instrument,
                                          station, year)
            # thg_l2_mag_amd_20170109_v01.cdf
            remote_file = remote_path + '/' + file_dir + '/' + filename
            local_file = os.path.join(data_dir, file_dir_local, filename)

            file_list.append((remote_file, local_file))

    return file_list


def gmag_load(dates, stations, group=None, downloadonly=False, varformat=None,
              get_support_data=False, prefix='', suffix=''):
    """Loads themis data into pytplot variables"""

    if group is not None:
        stations = gmag_list(group)

    file_list = gmag_filename(dates, stations)

    count = 0
    dcount = 0
    for remotef, localf in file_list:
        count += 1
        resp, err, locafile = pyspedas.download_files(remotef, localf)
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + locafile)
            dcount += 1
            if not downloadonly:
                try:
                    testcdf = pyspedas.cdf_to_tplot(locafile, varformat,
                                                    get_support_data, prefix,
                                                    suffix, False, True)
                    print(testcdf)
                except TypeError as e:
                    msg = "cdf_to_tplot could not load " + locafile
                    + "\nError:\n" + str(e)
                    print(msg)

        else:
            print(str(count) + '. There was a problem. Could not download \
                  file: ' + remotef)
            print(err)

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pyspedas.tplot_names())



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open

setup(
    name='pyspedas',
    version='0.8.23',
    description='Python Space Physics Environment Data Analysis \
                    Software (SPEDAS)',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/spedas/pyspedas',
    author='Nick Hatzigeorgiu, Eric Grimes',
    author_email='nikos@berkeley.edu',
    license='MIT',
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Science/Research',
                 'Topic :: Scientific/Engineering',
                 'License :: OSI Approved :: MIT License',
                 'Programming Language :: Python :: 3',
                 ],
    keywords='spedas data tools',
    project_urls={'Information': 'http://spedas.org/wiki/',
                  },
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['requests', 'pytplot', 'cdflib', 'bokeh', 'nodejs'],
    python_requires='>=3.5',
)



"""
This module contains routines for loading MAVEN data.
"""

from .maven_load import load_data


def maven_load(filenames=None,
               instruments=None,
               level='l2',
               insitu=True,
               iuvs=False,
               start_date='2014-01-01',
               end_date='2020-01-01',
               update_prefs=False,
               only_update_prefs=False,
               local_dir=None,
               list_files=False,
               new_files=False,
               exclude_orbit_file=False,
               download_only=False,
               varformat=None,
               prefix='',
               suffix='',
               get_support_data=False):
    """
    Main function for downloading MAVEN data and loading it into tplot variables (if applicable).

    Parameters:
        filenames: str/list of str ['yyyy-mm-dd']
            List of dates to be downloaded (eg. ['2015-12-31']).
        instruments: str/list of str
            Instruments from which you want to download data.
        list_files: bool (True/False0
            If true, lists the files instead of downloading them.
        level: str
            Data level to download.
        insitu: bool (True/False)
            If true, specifies only insitu files.
        iuvs: bool (True/False)
            If true,
        new_files: bool (True/False)
            Checks downloaded files and only downloads those that haven't already been downloaded.
        start_date: str
            String that is the start date for downloading data (YYYY-MM-DD)
        end_date: str
            String that is the end date for downloading data (YYYY-MM-DD)
        update_prefs: bool (True/False)
            If true, updates where you want to store data locally
        only_update_prefs: bool (True/False)
            If true, *only* updates where to store dat alocally, doesn't download files.
        exclude_orbit_file: bool (True/False)
            If true, won't download the latest orbit tables.
        local_dir: str
            If indicated, specifies where to download files for a specific implementation of this function.
        download_only: bool (True/False)
            If True then files are downloaded only,
            if False then CDF files are also loaded into pytplot using cdf_to_tplot.
        varformat : str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.
        prefix: str
            The tplot variable names will be given this prefix.
            By default, no prefix is added.
        suffix: str
            The tplot variable names will be given this suffix.
            By default, no suffix is added.
        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".
    """
    tvars = load_data(filenames=filenames, instruments=instruments, level=level, insitu=insitu, iuvs=iuvs,
                      start_date=start_date, end_date=end_date, update_prefs=update_prefs,
                      only_update_prefs=only_update_prefs, local_dir=local_dir, list_files=list_files,
                      new_files=new_files, exclude_orbit_file=exclude_orbit_file, download_only=download_only,
                      varformat=varformat, prefix=prefix, suffix=suffix, get_support_data=get_support_data)
    return tvars



'''

 Purpose:
    this function returns the FEEPS active eyes,
    based on date/probe/species/rate

 Output:
    Returns a hash table containing 2 hash tables:
       output['top'] -> maps to the active top eyes
       output['bottom'] -> maps to the active bottom eyes
 
 Notes:
 1) Burst mode should include all sensors (TOP and BOTTOM):
     electrons: [1, 2, 3, 4, 5, 9, 10, 11, 12]
     ions: [6, 7, 8]
 
 2) SITL should return (TOP only):
     electrons: set_intersection([5, 11, 12], active_eyes)
     ions: None
     
 3) From Drew Turner, 9/7/2017, srvy mode:
 
   - before 16 August 2017:
      electrons: [3, 4, 5, 11, 12]
      iond: [6, 7, 8]
 
   - after 16 August 2017:
       MMS1
         Top Eyes: 3, 5, 6, 7, 8, 9, 10, 12
         Bot Eyes: 2, 4, 5, 6, 7, 8, 9, 10

       MMS2
         Top Eyes: 1, 2, 3, 5, 6, 8, 10, 11
         Bot Eyes: 1, 4, 5, 6, 7, 8, 9, 11

       MMS3
         Top Eyes: 3, 5, 6, 7, 8, 9, 10, 12
         Bot Eyes: 1, 2, 3, 6, 7, 8, 9, 10

       MMS4
         Top Eyes: 3, 4, 5, 6, 8, 9, 10, 11
         Bot Eyes: 3, 5, 6, 7, 8, 9, 10, 12

'''

from pyspedas import time_double

def mms_feeps_active_eyes(trange, probe, data_rate, species, level):
    if data_rate.lower() == 'brst' and species.lower() == 'electron': return {'top': [1, 2, 3, 4, 5, 9, 10, 11, 12], 'bottom': [1, 2, 3, 4, 5, 9, 10, 11, 12]}
    if data_rate.lower() == 'brst' and species.lower() == 'ion': return {'top': [6, 7, 8], 'bottom': [6, 7, 8]}

    # old eyes, srvy mode, prior to 16 August 2017
    if species.lower() == 'electron':
        sensors = {'top': [3, 4, 5, 11, 12], 'bottom': [3, 4, 5, 11, 12]}
    else:
        sensors = {'top': [6, 7, 8], 'bottom': [6, 7, 8]}

    # srvy mode, after 16 August 2017
    if time_double(trange[0]) >= time_double('2017-08-16') and data_rate.lower() == 'srvy':
        active_table = {'1-electron': {'top': [3, 5, 9, 10, 12], 'bottom': [2, 4, 5, 9, 10]}}
        active_table = {'1-ion': {'top': [6, 7, 8], 'bottom': [6, 7, 8]}}
        active_table = {'2-electron': {'top': [1, 2, 3, 5, 10, 11], 'bottom': [1, 4, 5, 9, 11]}}
        active_table = {'2-ion': {'top': [6, 8], 'bottom': [6, 7, 8]}}
        active_table = {'3-electron': {'top': [3, 5, 9, 10, 12], 'bottom': [1, 2, 3, 9, 10]}}
        active_table = {'3-ion': {'top': [6, 7, 8], 'bottom': [6, 7, 8]}}
        active_table = {'4-electron': {'top': [3, 4, 5, 9, 10, 11], 'bottom': [3, 5, 9, 10, 12]}}
        active_table = {'4-ion': {'top': [6, 8], 'bottom': [6, 7, 8]}}
        sensors = active_table[probe.lower()+'-'+species.lower()]
        if level.lower() == 'sitl':
            return {'top': list(set(sensors['top']) & set([5, 11, 12])), 'bottom': []}

    if level.lower() == 'sitl':
        return {'top': [5, 11, 12], 'bottom': []}

    return sensors





# -*- coding: utf-8 -*-
"""
File:
    helpers.py

Description:
    Helper functions for pyspedas.
"""

import urllib
import os
import socket
import warnings


def download_files(url, locafile):
    """Get a file from internet and save it localy."""

    warnings.simplefilter("ignore", ResourceWarning)
    
    exists = False
    httpreq = None
    err = None
    ver = -1

    exists, err, ver, newurl = find_latest_url_version(url)
    if not exists:
        return exists, err, locafile

    url = newurl
    if ver != -1:
        locafile = locafile.replace('?', str(ver))

    # Create local directory if it does not exist
    dirPath = os.path.dirname(locafile)
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)

    # Download file
    try:
        if not os.path.exists(locafile):
            httpreq = urllib.request.urlretrieve(url, locafile)
    except urllib.request.URLError as e:
        httpreq = None
        err = e
    except socket.error:
        pass

    if httpreq is not None:
        exists = True

    return exists, err, locafile


def url_exists(url):
    """Returns True if url exists, otherwise False"""

    ans = False
    try:
        hreq = urllib.request.urlopen(url)
        ans = True
        err = ''
        hreq.close()
    except urllib.request.URLError as e:
        ans = False
        err = e

    return ans, err


def find_latest_url_version(url):
    """Returns if a file exists and the latest version.
    Assumes max version = v04, should be changed if there are higher versions
    """

    exists = False
    err = ''
    ver = -1
    newurl = url
    if '?' in url:
        for ver in range(4, -1, -1):
            newurl = url.replace('?', str(ver))
            exists, err = url_exists(newurl)
            # print(exists, err, ver, newurl)
            if exists:
                break
    else:
        ver = -1
        newurl = url
        exists, err = url_exists(newurl)
        return exists, err, ver, newurl

    return exists, err, ver, newurl
